@@ -16,8 +16,8 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifndef UPSKIRT_MARKDOWN_H
-#define UPSKIRT_MARKDOWN_H
+#ifndef MARKDOWN_H__
+#define MARKDOWN_H__
 
 #include "buffer.h"
 #include "autolink.h"
@@ -26,11 +26,6 @@
 extern "C" {
 #endif
 
-#define SUNDOWN_VERSION "1.16.0"
-#define SUNDOWN_VER_MAJOR 1
-#define SUNDOWN_VER_MINOR 16
-#define SUNDOWN_VER_REVISION 0
-
 /********************
  * TYPE DEFINITIONS *
  ********************/
@@ -61,7 +56,9 @@ enum mkd_extensions {
 	MKDEXT_SUPERSCRIPT = (1 << 7),
 	MKDEXT_LAX_SPACING = (1 << 8),
 	MKDEXT_DISABLE_INDENTED_CODE = (1 << 9),
-	MKDEXT_HIGHLIGHT = (1 << 10)
+	MKDEXT_HIGHLIGHT = (1 << 10),
+	MKDEXT_FOOTNOTES = (1 << 11),
+	MKDEXT_QUOTE = (1 << 12)
 };
 
 /* sd_callbacks - functions for rendering parsed data */
@@ -78,7 +75,8 @@ struct sd_callbacks {
 	void (*table)(struct buf *ob, const struct buf *header, const struct buf *body, void *opaque);
 	void (*table_row)(struct buf *ob, const struct buf *text, void *opaque);
 	void (*table_cell)(struct buf *ob, const struct buf *text, int flags, void *opaque);
-
+	void (*footnotes)(struct buf *ob, const struct buf *text, void *opaque);
+	void (*footnote_def)(struct buf *ob, const struct buf *text, unsigned int num, void *opaque);
 
 	/* span level callbacks - NULL or return 0 prints the span verbatim */
 	int (*autolink)(struct buf *ob, const struct buf *link, enum mkd_autolink type, void *opaque);
@@ -87,6 +85,7 @@ struct sd_callbacks {
 	int (*emphasis)(struct buf *ob, const struct buf *text, void *opaque);
 	int (*underline)(struct buf *ob, const struct buf *text, void *opaque);
 	int (*highlight)(struct buf *ob, const struct buf *text, void *opaque);
+	int (*quote)(struct buf *ob, const struct buf *text, void *opaque);
 	int (*image)(struct buf *ob, const struct buf *link, const struct buf *title, const struct buf *alt, void *opaque);
 	int (*linebreak)(struct buf *ob, void *opaque);
 	int (*link)(struct buf *ob, const struct buf *link, const struct buf *title, const struct buf *content, void *opaque);
@@ -94,6 +93,7 @@ struct sd_callbacks {
 	int (*triple_emphasis)(struct buf *ob, const struct buf *text, void *opaque);
 	int (*strikethrough)(struct buf *ob, const struct buf *text, void *opaque);
 	int (*superscript)(struct buf *ob, const struct buf *text, void *opaque);
+	int (*footnote_ref)(struct buf *ob, unsigned int num, void *opaque);
 
 	/* low level callbacks - NULL copies input directly into the output */
 	void (*entity)(struct buf *ob, const struct buf *entity, void *opaque);
@@ -104,6 +104,9 @@ struct sd_callbacks {
 	void (*doc_footer)(struct buf *ob, void *opaque);
 };
 
+/* header methods used internally in Redcarpet */
+char* header_anchor(struct buf *text);
+
 struct sd_markdown;
 
 /*********
@@ -131,9 +134,6 @@ sd_markdown_render(struct buf *ob, const uint8_t *document, size_t doc_size, str
 extern void
 sd_markdown_free(struct sd_markdown *md);
 
-extern void
-sd_version(int *major, int *minor, int *revision);
-
 #ifdef __cplusplus
 }
 #endif
