@@ -1,4 +1,3 @@
-
 module Redcarpet
   module Render
     # Markdown-stripping renderer. Turns Markdown into plaintext
@@ -16,6 +15,9 @@ module Redcarpet
         :triple_emphasis, :strikethrough,
         :superscript,
 
+        # footnotes
+        :footnotes, :footnote_def, :footnote_ref,
+
         # low level rendering
         :entity, :normal_text
       ].each do |method|
@@ -29,6 +31,11 @@ module Redcarpet
         content
       end
 
+      def image(link, title, content)
+        content &&= content + " "
+        "#{content}#{link}"
+      end
+
       def paragraph(text)
         text + "\n"
       end
