@@ -67,6 +67,27 @@ class MarkdownTest < Test::Unit::TestCase
       "<ul>\n<li>Blah</li>\n</ul>\n", markdown
   end
 
+  # https://github.com/vmg/redcarpet/issues/111
+  def test_p_with_less_than_4space_indent_should_not_be_part_of_last_list_item
+    text = <<MARKDOWN
+  * a
+  * b
+  * c
+
+  This paragraph is not part of the list.
+MARKDOWN
+    expected = <<HTML
+<ul>
+<li>a</li>
+<li>b</li>
+<li>c</li>
+</ul>
+
+<p>This paragraph is not part of the list.</p>
+HTML
+    html_equal expected, @markdown.render(text)
+  end
+
   # http://github.com/rtomayko/rdiscount/issues/#issue/13
   def test_headings_with_trailing_space
     text = "The Ant-Sugar Tales \n"         +
@@ -179,6 +200,15 @@ EOS
     assert output.include? '<mark>highlighted</mark>'
   end
 
+  def test_quote_flag_works
+    text = 'this is "quote"'
+
+    refute render_with({}, text).include? '<q>quote</q>'
+
+    output = render_with({:quote => true}, text)
+    assert output.include? '<q>quote</q>'
+  end
+
   def test_that_fenced_flag_works
     text = <<fenced
 This is a simple test
@@ -203,12 +233,6 @@ fenced
     assert !out.include?("<pre><code>")
   end
 
-  def test_that_prettify_works
-    text = "foo\nbar\n```\nsome\ncode\n```\nbaz"
-    out = Redcarpet::Markdown.new(Redcarpet::Render::HTML.new(:prettify => true), :fenced_code_blocks => true).render(text)
-    assert !out.include?("<pre><code class=\"prettyprint\">")
-  end
-
   def test_that_indented_flag_works
     text = <<indented
 This is a simple text
@@ -245,9 +269,27 @@ text
     assert render_with({:no_intra_emphasis => true}, "this fails: hello_world_") !~ /<em>/
     assert render_with({:no_intra_emphasis => true}, "this also fails: hello_world_#bye") !~ /<em>/
     assert render_with({:no_intra_emphasis => true}, "this works: hello_my_world") !~ /<em>/
+    assert render_with({:no_intra_emphasis => true}, "句中**粗體**測試") =~ /<strong>/
 
     markdown = "This is (**bold**) and this_is_not_italic!"
     html = "<p>This is (<strong>bold</strong>) and this_is_not_italic!</p>\n"
     assert_equal html, render_with({:no_intra_emphasis => true}, markdown)
+
+    markdown = "This is \"**bold**\""
+    html = "<p>This is &quot;<strong>bold</strong>&quot;</p>\n"
+    assert_equal html, render_with({:no_intra_emphasis => true}, markdown)
+  end
+
+  def test_ordered_lists_with_lax_spacing
+    markdown = "Foo:\n1. Foo\n2. Bar"
+    output = render_with({lax_spacing: true}, markdown)
+
+    assert_match /<ol>/, output
+    assert_match /<li>Foo<\/li>/, output
+  end
+
+  def test_references_with_tabs_after_colon
+    markdown = @markdown.render("[Link][id]\n[id]:\t\t\thttp://google.es")
+    html_equal '<p><a href="http://google.es">Link</a></p>', markdown
   end
 end
